@@ -238,7 +238,8 @@ class Generator:
             else:
                 firstTimeCalled = False
 
-            msg = '%s GENERATOR: Starting generation for %s' % (self._mode.upper(), obj)
+            msg = '%s GENERATOR: Starting %s generation for %s (%s)' % \
+                (self._mode.upper(), args.get('formatType'), obj, args.get('role'))
             debug.println(debug.LEVEL_INFO, msg, True)
 
             # Reset 'usedDescriptionForName' if a previous generator used it.
@@ -282,7 +283,7 @@ class Generator:
 
     def _resultElementToString(self, element, includeAll=True):
         if not includeAll:
-            return str(element)
+            return str(element).replace("\n", "\\n")
 
         return "\n%s'%s'" % (" " * 18, element)
 
@@ -1297,6 +1298,12 @@ class Generator:
             return 'ROLE_CONTENT_MARK'
         if self._script.utilities.isContentSuggestion(obj):
             return 'ROLE_CONTENT_SUGGESTION'
+        if self._script.utilities.isDescriptionList(obj):
+            return pyatspi.ROLE_DESCRIPTION_LIST
+        if self._script.utilities.isDescriptionListTerm(obj):
+            return pyatspi.ROLE_DESCRIPTION_TERM
+        if self._script.utilities.isDescriptionListDescription(obj):
+            return pyatspi.ROLE_DESCRIPTION_VALUE
         if self._script.utilities.isLandmark(obj):
             if self._script.utilities.isLandmarkRegion(obj):
                 return 'ROLE_REGION'
