@@ -121,6 +121,41 @@ _restoreOrcaKeys = False
 #                                                                      #
 ########################################################################
 
+CARET_TRACKING = "caret-tracking"
+FOCUS_TRACKING = "focus-tracking"
+FLAT_REVIEW = "flat-review"
+MOUSE_REVIEW = "mouse-review"
+SAY_ALL = "say-all"
+
+def emitRegionChanged(obj, startOffset=None, endOffset=None, mode=None):
+    """Notifies interested clients that the current region of interest has changed."""
+
+    if startOffset is None:
+        startOffset = 0
+    if endOffset is None:
+        endOffset = startOffset
+    if mode is None:
+        mode = FOCUS_TRACKING
+
+    try:
+        obj.emit("mode-changed::" + mode, 1, "")
+    except:
+        msg = "ORCA: Exception emitting mode-changed notification"
+        debug.println(debug.LEVEL_INFO, msg, True)
+
+    if mode != orca_state.activeMode:
+        msg = "ORCA: Switching active mode from %s to %s" % (orca_state.activeMode, mode)
+        debug.println(debug.LEVEL_INFO, msg, True)
+        orca_state.activeMode = mode
+
+    try:
+        msg = "ORCA: Region of interest: %s (%i, %i)" % (obj, startOffset, endOffset)
+        debug.println(debug.LEVEL_INFO, msg, True)
+        obj.emit("region-changed", startOffset, endOffset)
+    except:
+        msg = "ORCA: Exception emitting region-changed notification"
+        debug.println(debug.LEVEL_INFO, msg, True)
+
 def setLocusOfFocus(event, obj, notifyScript=True, force=False):
     """Sets the locus of focus (i.e., the object with visual focus) and
     notifies the script of the change should the script wish to present
@@ -223,6 +258,14 @@ def _processBrailleEvent(event):
 #                                                                      #
 ########################################################################
 
+def deviceChangeHandler(deviceManager, device):
+    """New keyboards being plugged in stomp on our changes to the keymappings, so we have to re-apply"""
+    source = device.get_source()
+    if source == Gdk.InputSource.KEYBOARD:
+        msg = "ORCA: Keyboard change detected, re-creating the xmodmap"
+        debug.println(debug.LEVEL_INFO, msg, True)
+        _createOrcaXmodmap()
+
 def updateKeyMap(keyboardEvent):
     """Unsupported convenience method to call sad hacks which should go away."""
 
@@ -336,6 +379,11 @@ def _restoreXmodmap(keyList=[]):
         stdin=subprocess.PIPE, stdout=None, stderr=None)
     p.communicate(_originalXmodmap)
 
+def setKeyHandling(new):
+    """Toggle use of the new vs. legacy key handling mode.
+    """
+    _eventManager.setKeyHandling(new)
+
 def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     """Loads (and reloads) the user settings module, reinitializing
     things such as speech if necessary.
@@ -382,6 +430,8 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     _settingsManager.loadAppSettings(script)
 
     if _settingsManager.getSetting('enableSpeech'):
+        msg = 'ORCA: About to enable speech'
+        debug.println(debug.LEVEL_INFO, msg, True)
         try:
             speech.init()
             if reloaded and not skipReloadMessage:
@@ -393,12 +443,18 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
         debug.println(debug.LEVEL_INFO, msg, True)
 
     if _settingsManager.getSetting('enableBraille'):
+        msg = 'ORCA: About to enable braille'
+        debug.println(debug.LEVEL_INFO, msg, True)
         try:
-            braille.init(_processBrailleEvent, settings.tty)
+            braille.init(_processBrailleEvent)
         except:
             debug.printException(debug.LEVEL_WARNING)
             msg = 'ORCA: Could not initialize connection to braille.'
             debug.println(debug.LEVEL_WARNING, msg, True)
+    else:
+        msg = 'ORCA: Braille is not enabled in settings'
+        debug.println(debug.LEVEL_INFO, msg, True)
+
 
     if _settingsManager.getSetting('enableMouseReview'):
         mouse_review.reviewer.activate()
@@ -490,6 +546,20 @@ def helpForOrca(script=None, inputEvent=None, page=""):
                  Gtk.get_current_event_time())
     return True
 
+def addKeyGrab(binding):
+    """ Add a key grab for the given key binding. """
+    ret = []
+    for kd in binding.keyDefs():
+        ret.append(orca_state.device.add_key_grab(kd, None))
+    return ret
+
+def removeKeyGrab(id):
+    """ Remove the key grab for the given key binding. """
+    orca_state.device.remove_key_grab(id)
+
+def mapModifier(keycode):
+    return orca_state.device.map_modifier(keycode)
+
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
     If so, show the confirmation GUI otherwise just shutdown.
@@ -531,6 +601,7 @@ def init(registry):
     global _initialized
 
     if _initialized and _settingsManager.isScreenReaderServiceEnabled():
+        debug.println(debug.LEVEL_INFO, 'ORCA: Already initialized', True)
         return False
 
     # Do not hang on initialization if we can help it.
@@ -556,8 +627,7 @@ def init(registry):
     return True
 
 def start(registry, cacheValues):
-    """Starts Orca.
-    """
+    """Starts Orca."""
 
     debug.println(debug.LEVEL_INFO, 'ORCA: Starting', True)
 
@@ -576,6 +646,15 @@ def start(registry, cacheValues):
     if cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
+    # Event handlers for input devices being plugged in/unplugged.
+    # Used to re-create the Xmodmap when a new keyboard is plugged in.
+    # Necessary, because plugging in a new keyboard resets the Xmodmap
+    # and stomps our changes
+    display = Gdk.Display.get_default()
+    devmanager=display.get_device_manager()
+    devmanager.connect("device-added", deviceChangeHandler)
+    devmanager.connect("device-removed", deviceChangeHandler)
+
     Gdk.notify_startup_complete()
     msg = 'ORCA: Startup complete notification made'
     debug.println(debug.LEVEL_INFO, msg, True)
@@ -598,8 +677,8 @@ def die(exitCode=1):
 def timeout(signum=None, frame=None):
     msg = 'TIMEOUT: something has hung. Aborting.'
     debug.println(debug.LEVEL_SEVERE, msg, True)
-    debug.printStack(debug.LEVEL_ALL)
-    debug.examineProcesses()
+    debug.printStack(debug.LEVEL_SEVERE)
+    debug.examineProcesses(force=True)
     die(EXIT_CODE_HANG)
 
 def shutdown(script=None, inputEvent=None):
@@ -622,7 +701,7 @@ def shutdown(script=None, inputEvent=None):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
         signal.alarm(settings.timeoutTime)
 
-    orca_state.activeScript.presentMessage(messages.STOP_ORCA)
+    orca_state.activeScript.presentMessage(messages.STOP_ORCA, resetStyles=False)
 
     _scriptManager.deactivate()
     _eventManager.deactivate()
@@ -654,7 +733,14 @@ exitCount = 0
 def shutdownOnSignal(signum, frame):
     global exitCount
 
-    msg = 'ORCA: Shutting down and exiting due to signal=%d' % signum
+    try:
+        # Requires python 3.8
+        signalString = '(%s)' % signal.strsignal(signum)
+    except:
+        signalString = ''
+
+    msg = 'ORCA: Shutting down and exiting due to signal=%d %s' % \
+        (signum, signalString)
     debug.println(debug.LEVEL_INFO, msg, True)
 
     # Well...we'll try to exit nicely, but if we keep getting called,
@@ -702,8 +788,13 @@ def main(cacheValues=True):
     an exit code of 0 means normal completion and an exit code of 50
     means Orca exited because of a hang."""
 
+    msg = "ORCA: Launching version %s" % orca_platform.version
+    if orca_platform.revision:
+        msg += " (rev %s)" % orca_platform.revision
+    debug.println(debug.LEVEL_INFO, msg, True)
+
     if debug.debugFile and os.path.exists(debug.debugFile.name):
-        faulthandler.enable(file=debug.debugFile, all_threads=False)
+        faulthandler.enable(file=debug.debugFile, all_threads=True)
     else:
         faulthandler.enable(all_threads=False)
 
@@ -719,10 +810,13 @@ def main(cacheValues=True):
     signal.signal(signal.SIGQUIT, shutdownOnSignal)
     signal.signal(signal.SIGSEGV, crashOnSignal)
 
+    debug.println(debug.LEVEL_INFO, "ORCA: Enabling accessibility (if needed).", True)
     if not _settingsManager.isAccessibilityEnabled():
         _settingsManager.setAccessibility(True)
 
+    debug.println(debug.LEVEL_INFO, "ORCA: Initializing ATSPI registry.", True)
     init(pyatspi.Registry)
+    debug.println(debug.LEVEL_INFO, "ORCA: ATSPI registry initialized.", True)
 
     try:
         message = messages.START_ORCA
@@ -743,14 +837,19 @@ def main(cacheValues=True):
             else:
                 script = _scriptManager.getScript(app, window)
                 _scriptManager.setActiveScript(script, "Launching.")
+
             setLocusOfFocus(None, window)
             focusedObject = script.utilities.focusedObject(window)
             if focusedObject:
                 setLocusOfFocus(None, focusedObject)
+                script = _scriptManager.getScript(focusedObject.getApplication(), focusedObject)
+                _scriptManager.setActiveScript(script, "Found focused object.")
 
     try:
+        debug.println(debug.LEVEL_INFO, "ORCA: Starting ATSPI registry.", True)
         start(pyatspi.Registry, cacheValues) # waits until we stop the registry
     except:
+        debug.println(debug.LEVEL_SEVERE, "ORCA: Exception starting ATSPI registry.", True)
         die(EXIT_CODE_HANG)
     return 0
 
