@@ -58,6 +58,8 @@ class ScriptManager:
              'metacity': 'switcher',
              'pluma': 'gedit',
             }
+        self._toolkitNames = \
+            {'WebKitGTK': 'WebKitGtk'}
 
         self.setActiveScript(None, "__init__")
         self._desktop = pyatspi.Registry.getDesktop(0)
@@ -134,6 +136,7 @@ class ScriptManager:
             else:
                 attrs = dict([attr.split(':', 1) for attr in attributes])
                 name = attrs.get('toolkit', '')
+                name = self._toolkitNames.get(name, name)
 
         return name
 
@@ -340,17 +343,33 @@ class ScriptManager:
 
         def _pid(app):
             try:
-                return app.get_process_id()
+                result = app.get_process_id()
+                msg = "SCRIPT MANAGER: %s is has pid: %s" % (app, result)
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return result
             except:
+                msg = "SCRIPT MANAGER: Exception getting pid for %s" % app
+                debug.println(debug.LEVEL_INFO, msg, True)
                 return -1
 
+        def _isValidApp(app):
+            try:
+                result = app in self._desktop
+                msg = "SCRIPT MANAGER: %s is in desktop: %s" % (app, result)
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return result
+            except:
+                msg = "SCRIPT MANAGER: Exception seeing if %s is in desktop" % app
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return False
+
         pid = _pid(app)
         if pid == -1:
             return None
 
         items = self.appScripts.items()
         for a, script in items:
-            if a != app and _pid(a) == pid and a in self._desktop:
+            if a != app and _pid(a) == pid and _isValidApp(a):
                 return script
 
         return None
@@ -371,7 +390,16 @@ class ScriptManager:
             msg = "SCRIPT MANAGER: %s is no longer in registry's desktop" % app
             debug.println(debug.LEVEL_INFO, msg, True)
 
-            appScript = self.appScripts.pop(app)
+            try:
+                appScript = self.appScripts.pop(app)
+            except KeyError:
+                msg = "SCRIPT MANAGER: %s not found in appScripts" % app
+                debug.println(debug.LEVEL_INFO, msg, True)
+                continue
+
+            msg = "SCRIPT MANAGER: Old script for app found: %s" % appScript
+            debug.println(debug.LEVEL_INFO, msg, True)
+
             newScript = self._getScriptForAppReplicant(app)
             if newScript:
                 msg = "SCRIPT MANAGER: Script for app replicant found: %s" % newScript
